#include "Dummy File Maker.h"

// Global variables
HICON g_hIcon;	// Icon handle for main and about dialog box.
HWND g_hwndMainDlg; // Window handle of main dialog box.
HINSTANCE g_hInstance;	// Application instance handle.
BOOL g_bCancel=FALSE;	// Cancel flag.

// Entry point 
int APIENTRY  WinMain(HINSTANCE hInstance,HINSTANCE hPrevInstance,LPSTR lpCmdLine,int nCmdShow)
{
	// Register progress bar control class from 
	// the common control dynamic-link library (COMCTL32.DLL). 
	INITCOMMONCONTROLSEX init_cc_ex = {
		sizeof init_cc_ex, ICC_PROGRESS_CLASS,};
		InitCommonControlsEx( &init_cc_ex );

		// Load icon for main and about dialog box.
		g_hIcon=LoadIcon(hInstance, MAKEINTRESOURCE(IDI_ICON) );

		// Save Application instance handle for later use in other functions.
		g_hInstance=hInstance;

		// Create main dialog box.
		DialogBox(hInstance, (LPCTSTR)IDD_MAIN, 0, (DLGPROC)DlgProcMain);

		// Clean up
		DestroyIcon(g_hIcon);
		return 0;
}

// Main dialog box messages handler.
LRESULT CALLBACK DlgProcMain(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{

	switch (uMsg)
	{
	case WM_COMMAND:
		if (HIWORD(wParam) == BN_CLICKED)// On button click 
		{
			switch(LOWORD(wParam))
			{
			case IDC_CHK_EMPTY:// On click "Create Empty File" check box
				// if "Create Empty File" check box is checked then disable size box.
				if (IsDlgButtonChecked(hwndDlg,IDC_CHK_EMPTY)==BST_CHECKED)
					EnableSizeBox(FALSE);
				else
					EnableSizeBox(TRUE);
				break;
			case IDC_BTN_FILE_NAME:
				// Get a new file name using save dialog.
				GetFileName();
				break;
			case IDC_BTN_ABOUT:
				// Show about dialog box.
				DialogBox(g_hInstance, (LPCTSTR)IDD_ABOUT, 0, (DLGPROC)DlgProcAbout);
				break;
			case IDC_BTN_CREATE:
				{
					// Create new file by creating new thread.
					// CreateThread call OnCreateButton function.
					DWORD dwThreadID;
					CreateThread(NULL, 0,
						(LPTHREAD_START_ROUTINE) OnCreateButton,
						NULL, 0, &dwThreadID);
				}
				break;

			}

		}
		break;
	case WM_INITDIALOG:
		// Save Main dialog box window handle for use in other function.
		g_hwndMainDlg=hwndDlg;
		// Set state of "Create Empty File" check box to checked.
		CheckDlgButton(hwndDlg,IDC_CHK_EMPTY,BST_CHECKED);
		// Set state of "Byte" radio button to checked.
		CheckDlgButton(hwndDlg,IDC_OPT_BYTES,BST_CHECKED);
		// Disable size box at startup.
		EnableSizeBox(FALSE);
		// Set main dialog box icon.
		SendMessage(hwndDlg,WM_SETICON,ICON_BIG,(LPARAM)g_hIcon);
		return true;
	case WM_CLOSE:
		EndDialog(hwndDlg,0);
		break;


	}
	return FALSE;
}
// About dialog related messages handler.
LRESULT CALLBACK DlgProcAbout(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	switch (uMsg)
	{
	case WM_COMMAND:
		if ( wParam==IDOK)// on OK button 
			EndDialog(hwndDlg,NULL);
		break;
	case WM_INITDIALOG:
		{	
			// set about dialog box icon
			SendMessage(hwndDlg,WM_SETICON,ICON_BIG,(LPARAM)g_hIcon);
			return true;
		}
	case WM_CLOSE:
		EndDialog(hwndDlg,NULL);
		break;

	}
	return FALSE;
}
// Enable or Disable Size box
void EnableSizeBox(BOOL bEnable)
{
	// "Bytes" radio button.
	EnableWindow(GetDlgItem(g_hwndMainDlg,IDC_OPT_BYTES),bEnable);
	// "KB" radio button.
	EnableWindow(GetDlgItem(g_hwndMainDlg,IDC_OPT_KB),bEnable);
	// "MB" radio button.
	EnableWindow(GetDlgItem(g_hwndMainDlg,IDC_OPT_MB),bEnable);
	// "File Size" text box.
	EnableWindow(GetDlgItem(g_hwndMainDlg,IDC_TXT_SIZE),bEnable);
	// "Fill with random bytes" check box.
	EnableWindow(GetDlgItem(g_hwndMainDlg,IDC_CHK_RANDOM),bEnable);

}
// Get a file name/path.
void GetFileName(void)
{
	OPENFILENAME ofn;
	char szFile[MAX_PATH];

	ZeroMemory(&ofn, sizeof(ofn));
	szFile[0]=0;

	ofn.lpstrFile = szFile;
	ofn.lStructSize = sizeof(ofn);
	ofn.hwndOwner = g_hwndMainDlg;
	ofn.lpstrFile[0] = '\0';
	ofn.nMaxFile = sizeof(szFile);
	ofn.nFilterIndex = 1;
	ofn.lpstrFileTitle = NULL;
	ofn.nMaxFileTitle = 0;
	ofn.lpstrInitialDir = NULL;
	ofn.lpstrFilter ="All Files (*.*)\0*.*\0\0";
	ofn.Flags = OFN_NODEREFERENCELINKS;
	ofn.lpstrTitle="Save As...";
	ofn.lpstrDefExt="bin";// if user not type extension then "bin" is default extension.

	if (GetSaveFileName(&ofn)==TRUE)
		SetWindowText(GetDlgItem(g_hwndMainDlg,IDC_TXT_FILE_NAME),szFile);

}
// Try to create a new file.
void OnCreateButton(void)
{
	//chk
	char strCaption[16];
	GetDlgItemText(g_hwndMainDlg,IDC_BTN_CREATE,strCaption,sizeof(strCaption));
	// Check if "FillFile" function already running.
	if (lstrcmp(strCaption,"Cancel")==0)
	{
		g_bCancel=TRUE; // Cancel FillFile function by setting Cancel flag to TRUE.
		SetDlgItemText(g_hwndMainDlg,IDC_BTN_CREATE,"Create");
		return;
	}
	//~chk

	g_bCancel=FALSE;

	HANDLE hFile;
	DWORD dwLength;
	CHAR strBuffer[MAX_PATH];

	// Check if "Create Empty File" is Checked.
	// if checked then simply skip size options.
	if (IsDlgButtonChecked(g_hwndMainDlg,IDC_CHK_EMPTY)!=BST_CHECKED)
	{
		// try to get file size from text box.
		if (GetDlgItemText(g_hwndMainDlg,IDC_TXT_SIZE,strBuffer,sizeof(strBuffer)) < 1 )
		{
			ErrMsgBox("Please enter file size.");
			return;
		}
		// convert string to DWORD.
		dwLength = strtoul(strBuffer, NULL, 0);
		// check  for zero size.
		if (dwLength < 1)
		{
			ErrMsgBox("Please enter file size.");
			return;
		}
		// Check for  "KB" and "MB" radio button.
		if (IsDlgButtonChecked(g_hwndMainDlg,IDC_OPT_KB)==BST_CHECKED)
			dwLength *= 1024;// multiply file size 1 KB.
		if (IsDlgButtonChecked(g_hwndMainDlg,IDC_OPT_MB)==BST_CHECKED)
			dwLength *= 1048576;// multiply file size 1 MB.
	}

	// try to get file name/path.
	if (GetDlgItemText(g_hwndMainDlg,IDC_TXT_FILE_NAME,strBuffer,sizeof(strBuffer)) < 1 )
	{
		ErrMsgBox("Please enter/select file name.");
		return;
	}

	// Create new file.
	hFile = CreateFile(strBuffer,
		GENERIC_WRITE,FILE_SHARE_READ,NULL,
		CREATE_NEW,FILE_ATTRIBUTE_NORMAL,
		NULL);
	// check if new file is created.
	if (hFile==INVALID_HANDLE_VALUE)
	{
		ErrMsgBox("Cant create new file.");
		return;
	}
	// if "Create Empty File" check box is checked then 
	// show success message and close file.
	if (IsDlgButtonChecked(g_hwndMainDlg,IDC_CHK_EMPTY)==BST_CHECKED)
	{
		CloseHandle(hFile);
		MessageBox(g_hwndMainDlg,"File is created.","Success",MB_OK | MB_ICONINFORMATION);
		return;
	}
	// activate Cancel option. 
	SetDlgItemText(g_hwndMainDlg,IDC_BTN_CREATE,"Cancel");
	// if "Fill with random bytes" check box is checked
	// then call "FillFile" with BOOL bRnd=TURE.
	if (IsDlgButtonChecked(g_hwndMainDlg,IDC_CHK_RANDOM)==BST_CHECKED)
		FillFile(hFile,dwLength,GetDlgItem(g_hwndMainDlg,IDC_PROGRESS_BAR),TRUE);
	else
		FillFile(hFile,dwLength,GetDlgItem(g_hwndMainDlg,IDC_PROGRESS_BAR),FALSE);
	// disable cancel option.
	SetDlgItemText(g_hwndMainDlg,IDC_BTN_CREATE,"Create");
	// finally close file.
	CloseHandle(hFile);

}
// Show error message box.
void ErrMsgBox(LPSTR lpstrError)
{
	MessageBox(g_hwndMainDlg,lpstrError,"Error",MB_ICONHAND | MB_OK);
}

// Fill(Write) file with zeros or random bytes.
void FillFile(HANDLE hFile,DWORD Length,HWND hwndPB,BOOL bRnd)
{
	BOOL bSuccess;
	DWORD dwRem , dwWrite;
	CHAR  pWriteBuf[WRITE_BUFFER_SIZE];

	// zero whole write buffer.
	ZeroMemory(pWriteBuf,WRITE_BUFFER_SIZE);
	//Sets a random starting point.
	srand( (unsigned)time( NULL ) );
	// set rem(remaining) variable to length. 
	dwRem=Length;

	do
	{
		//rand
		if (bRnd==TRUE)
		{
			int i;
			// Fill buffer with random bytes.
			for ( i = 0 ; i < WRITE_BUFFER_SIZE ; i++ )
				pWriteBuf[i]=rand() % 255;

		}
		//~rand
		if(dwRem > WRITE_BUFFER_SIZE)
		{
			bSuccess=WriteFile(hFile,pWriteBuf,WRITE_BUFFER_SIZE,&dwWrite,0);
			dwRem -=dwWrite;
		}
		else if (dwRem <= WRITE_BUFFER_SIZE)
		{
			bSuccess=WriteFile(hFile,pWriteBuf,dwRem,&dwWrite,0);
			dwRem=0;
		}
		// set progress bar position by getting percentage of written bytes.
		SendMessage(hwndPB,PBM_SETPOS,GetPercent(Length - dwRem,Length),0);


	} while ( dwRem > 0 && bSuccess && g_bCancel == FALSE);

	// check for error and success
	if (bSuccess != 0 && g_bCancel == FALSE)
		MessageBox(g_hwndMainDlg,"File is created.","Success",MB_OK | MB_ICONINFORMATION);
	else if (bSuccess == 0 && g_bCancel == FALSE)
		ErrMsgBox("File is created but error during writing to file.");
	SendMessage(hwndPB,PBM_SETPOS,0,0);


}
// get percentage in int(integer) from DWORD values.
// note :: this function taken from "Copy File Part" Application.  
int GetPercent(DWORD dwNewValue,DWORD dwMax)
{
	double dMax,dNewValue,dRet;
	int iRet;

	dMax=dwMax;
	dNewValue=dwNewValue;

	dRet =dNewValue / dMax;
	dRet *=100;

	iRet=dRet; // ???

	return(iRet);
}